@@ -358,6 +358,16 @@ class CGATS(dict):
 				elif values and values[0] not in ('Comment:', 'Date:') and \
 				     len(line) >= 3 and not re.search("[^ 0-9A-Za-z]", line):
 					context = self.add_data(line)
+			if 0 in self and self[0].get("NORMALIZED_TO_Y_100") == "NO":
+				# Always normalize to Y = 100
+				reprstr = (self.filename or "<%s.%s instance at 0x%016x>" %
+											(self.__module__,
+											 self.__class__.__name__, id(self)))
+				if self[0].normalize_to_y_100():
+					safe_print("Normalized to Y = 100:", reprstr)
+				else:
+					safe_print("Warning: Could not normalize to Y = 100:",
+							   reprstr)
 			self.setmodified(False)
 
 	def __delattr__(self, name):
@@ -1699,10 +1709,10 @@ Transform {
 
 	def fix_zero_measurements(self, warn_only=False, logfile=safe_print):
 		"""
-		Fix (or warn about) zero measurements
+		Fix (or warn about) <= zero measurements
 		
 		If XYZ/Lab = 0, the sample gets removed. If only one component of
-		XYZ/Lab is 0, it gets fudged so that the component is nonzero
+		XYZ/Lab is <= 0, it gets fudged so that the component is nonzero
 		(because otherwise, Argyll's colprof will remove it, which can have bad
 		effects if it's an 'essential' sample)
 		
@@ -1714,57 +1724,57 @@ Transform {
 			cie_labels = []
 			for channel in color_rep[1]:
 				cie_labels.append(color_rep[1] + "_" + channel)
+				if color_rep[1] == "LAB":
+					# Only check L* for zero values
+					break
 			device_labels = []
 			for channel in color_rep[0]:
 				device_labels.append(color_rep[0] + "_" + channel)
-			errors = []
 			remove = []
 			for key, sample in data.iteritems():
 				cie_values = [sample[label] for label in cie_labels]
 				# Check if zero
 				if filter(lambda v: v, cie_values):
-					# Not zero. Check if one component zero
-					if 0.0 in cie_values:
+					# Not all zero. Check if some component(s) equal or below zero
+					if min(cie_values) <= 0:
 						for label in cie_labels:
-							if not sample[label]:
+							if sample[label] <= 0:
 								if warn_only:
 									if logfile:
-										logfile.write("Warning: Sample ID %i (RGB %s) has %s = 0!\n" %
+										logfile.write("Warning: Sample ID %i (%s %s) has %s <= 0!\n" %
 													  (sample.SAMPLE_ID,
+													   color_rep[0],
 													   " ".join(str(sample.queryv1(device_labels)).split()),
 													   label))
 								else:
 									# Fudge to be nonzero
 									sample[label] = 0.000001
 									if logfile:
-										logfile.write("Fudged sample ID %i (RGB %s) %s to be non-zero\n" %
+										logfile.write("Fudged sample ID %i (%s %s) %s to be non-zero\n" %
 													  (sample.SAMPLE_ID,
+													   color_rep[0],
 													   " ".join(str(sample.queryv1(device_labels)).split()),
 													   label))
 					continue
-				# Zero
+				# All zero
 				device_values = [sample[label] for label in device_labels]
-				device_sum = 0
-				for value in device_values:
-					if value > 0:
-						# Error on device values > 0
-						errors.append(sample)
-					device_sum += value
-				if not device_sum:
+				if not max(device_values):
 					# Skip device black
 					continue
 				if warn_only:
 					if logfile:
-						logfile.write("Warning: Sample ID %i (RGB %s) has %s = 0!\n" %
+						logfile.write("Warning: Sample ID %i (%s %s) has %s = 0!\n" %
 									  (sample.SAMPLE_ID,
+									   color_rep[0],
 									   " ".join(str(sample.queryv1(device_labels)).split()),
 									   color_rep[1]))
 				else:
 					# Queue sample for removal
 					remove.insert(0, sample)
 					if logfile:
-						logfile.write("Removed sample ID %i (RGB %s) with %s = 0\n" %
+						logfile.write("Removed sample ID %i (%s %s) with %s = 0\n" %
 									  (sample.SAMPLE_ID,
+									   color_rep[0],
 									   " ".join(str(sample.queryv1(device_labels)).split()),
 									   color_rep[1]))
 			for sample in remove:
@@ -1788,8 +1798,27 @@ Transform {
 							fixed += 1
 							break
 		return fixed
+
+	def normalize_to_y_100(self):
+		""" Scale XYZ values so that RGB 100 = Y 100 """
+		if "DATA" in self:
+			white_cie = self.get_white_cie()
+			if white_cie and "XYZ_Y" in white_cie:
+				white_Y = white_cie["XYZ_Y"]
+				if white_Y != 100:
+					self.add_keyword("LUMINANCE_XYZ_CDM2",
+									 "%.4f %.4f %.4f" % (white_cie["XYZ_X"],
+														 white_cie["XYZ_Y"],
+														 white_cie["XYZ_Z"]))
+					for sample in self.DATA.itervalues():
+						for label in "XYZ":
+							v = sample["XYZ_" + label]
+							sample["XYZ_" + label] = v / white_Y * 100
+				self.add_keyword("NORMALIZED_TO_Y_100", "YES")
+				return True
+		return False
 	
-	def quantize_device_values(self, bits=8):
+	def quantize_device_values(self, bits=8, quantizer=round):
 		""" Quantize device values to n bits """
 		q = 2 ** bits - 1.0
 		for data in self.queryv("DATA").itervalues():
@@ -1801,7 +1830,7 @@ Transform {
 			for labels in get_device_value_labels(color_rep):
 				for item in data.queryi(labels).itervalues():
 					for label in labels:
-						item[label] = round(item[label] / maxv * q) / q * maxv
+						item[label] = quantizer(item[label] / maxv * q) / q * maxv
 	
 	def scale_device_values(self, factor=100.0 / 255, color_rep=None):
 		""" Scales device values by multiplying with factor. """
