@@ -15,6 +15,7 @@ import ICCProfile as ICCP
 import config
 import localization as lang
 import worker
+from util_list import natsort_key_factory
 from wxTestchartEditor import TestchartEditor
 from wxwindows import BaseApp, BaseFrame, FileDrop, InfoDialog, wx
 from wxfixes import TempXmlResource
@@ -90,8 +91,9 @@ class ReportFrame(BaseFrame):
 						if not (path.lower().endswith(".ti2") and
 								basepath + ".cie" in paths):
 							history.append(path)
+				natsort_key = natsort_key_factory()
 				history = sorted(history, key=lambda path:
-											  os.path.basename(path).lower())
+											  natsort_key(os.path.basename(path)))
 				ctrl.SetHistory(history)
 			ctrl.SetMaxFontSize(11)
 			# Drop targets
@@ -197,7 +199,7 @@ class ReportFrame(BaseFrame):
 		if v != getcfg("measurement_report.trc_output_offset"):
 			setcfg("measurement_report.trc_output_offset", v)
 			self.mr_update_trc_control()
-			self.mr_show_trc_controls()
+			#self.mr_show_trc_controls()
 
 	def mr_trc_gamma_ctrl_handler(self, event):
 		try:
@@ -231,6 +233,12 @@ class ReportFrame(BaseFrame):
 			setcfg("measurement_report.trc_gamma_type", "b")
 			setcfg("measurement_report.trc_output_offset", 1.0)
 			self.mr_update_trc_controls()
+		else:
+			# Custom
+			# Have to use CallAfter, otherwise only part of the text will
+			# be selected (wxPython bug?)
+			wx.CallAfter(self.mr_trc_gamma_ctrl.SetFocus)
+			wx.CallLater(1, self.mr_trc_gamma_ctrl.SelectAll)
 		self.mr_show_trc_controls()
 		self.Thaw()
 
@@ -468,7 +476,17 @@ class ReportFrame(BaseFrame):
 													   % odata, self)
 									self.set_profile_ctrl_path(which)
 									return
-								self.XYZbpout = odata[0]
+								if odata[0][1]:
+									# Got above zero blackpoint from lookup
+									self.XYZbpout = odata[0]
+								else:
+									# Got zero blackpoint from lookup.
+									# Try chardata instead.
+									XYZbp = profile.get_chardata_bkpt()
+									if XYZbp:
+										self.XYZbpout = XYZbp
+									else:
+										self.XYZbpout = [0, 0, 0]
 					else:
 						# Profile selection has not changed
 						# Restore cached XYZbp values
@@ -651,7 +669,9 @@ class ReportFrame(BaseFrame):
 				   isinstance(self.simulation_profile.tags.get("gXYZ"),
 							  ICCP.XYZType) and
 				   isinstance(self.simulation_profile.tags.get("bXYZ"),
-							  ICCP.XYZType))
+							  ICCP.XYZType) and
+				   not isinstance(self.simulation_profile.tags.get("A2B0"),
+							  ICCP.LUT16Type))
 		##print enable5, self.XYZbpin, self.XYZbpout
 		self.mr_trc_label.Show(enable1 and enable5)
 		self.apply_none_ctrl.Show(enable1 and enable5)
